/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2014 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "stdafx.h"
#include "PuttyAgent.h"
#include "CygSock.h"

#include "PageantWnd.h"
#include "CygSockChannel.h"


// CCygSock

CCygSock::CCygSock()
{
}

CCygSock::~CCygSock()
{
	if(!m_strSockpath.IsEmpty())
		::DeleteFile(m_strSockpath);
}

BOOL CCygSock::Create(const CString &strSockpath)
{
	try
	{
		CFile sockfile(strSockpath, CFile::modeWrite | CFile::modeCreate);

		// TODO: Cygwin explicitely sets something here (set_file_attribute)
		// Check if everything is done correctly here
		SetFileAttributes(strSockpath, FILE_ATTRIBUTE_SYSTEM);

		unsigned short int *connSecretAsWord = (unsigned short int*)m_iConnectSecret;
		for(int word = 0; word < 8; word++)
			connSecretAsWord[word] = rand();

		CAsyncSocket::Create(0, SOCK_STREAM, FD_READ | FD_WRITE | FD_ACCEPT | FD_CLOSE, _T("127.0.0.1"));
		if(!Listen())
			return FALSE;

		if(!GetSockName(m_strSockaddr, m_uPort))
			return FALSE;

		this->m_strSockpath = strSockpath;

		CString strSockinf;
		strSockinf.Format(_T("!<socket >%d s %08x-%08x-%08x-%08x"), this->m_uPort,
			m_iConnectSecret[0], m_iConnectSecret[1], m_iConnectSecret[2], m_iConnectSecret[3]);

		sockfile.Write(strSockinf, strSockinf.GetLength() + 1);
	}
	catch(CFileException *e)
	{
		e->Delete();
		return FALSE;
	}

	return TRUE;
}

BOOL CCygSock::CheckSecret(const int *out)
{
	if(out[0] == m_iConnectSecret[0] &&
		out[1] == m_iConnectSecret[1] &&
		out[2] == m_iConnectSecret[2] &&
		out[3] == m_iConnectSecret[3])
		return TRUE;
	return FALSE;
}

void CCygSock::GetSecret(int *out)
{
	out[0] = m_iConnectSecret[0];
	out[1] = m_iConnectSecret[1];
	out[2] = m_iConnectSecret[2];
	out[3] = m_iConnectSecret[3];
}

void CCygSock::Close()
{
	CAsyncSocket::Close();
	if(!m_strSockpath.IsEmpty())
	{
		::DeleteFile(m_strSockpath);
		m_strSockpath.Empty();
	}
}

void CCygSock::SetAgentWnd(CPageantWnd *pAgentWnd)
{
	m_pAgentWnd = pAgentWnd;
}

void CCygSock::OnAccept(int nErrorCode)
{
	ASSERT( m_pAgentWnd != NULL );

	if(m_pAgentWnd && nErrorCode==0)
	{
		CCygSockChannel *newSock = new CCygSockChannel();
		newSock->m_sCygSock = this;
		newSock->SetAgentWnd(this->m_pAgentWnd);

		Accept(*newSock);

#ifdef _DEBUG
		CString strIp;
		UINT uPort;
		newSock->GetSockName(strIp, uPort);
		TRACE(_T("Accepted new connection from %d:%d\n"), strIp, uPort);
#endif
	}

	CAsyncSocket::OnAccept(nErrorCode);
}

