/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2014 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "stdafx.h"
#include "PuttyAgent.h"
#include "CygSockChannel.h"

#include "PageantWnd.h"

// CCygSockChannel

CCygSockChannel::CCygSockChannel()
	: m_iState(0)
{
	TRACE(_T("CCygSockChannel::CCygSockChannel: %x\n"), this->m_hSocket);
}

CCygSockChannel::~CCygSockChannel()
{
	TRACE(_T("CCygSockChannel::~CCygSockChannel: %x\n"), this->m_hSocket);
}

void CCygSockChannel::SetAgentWnd(CPageantWnd *pAgentWnd)
{
	m_pAgentWnd = pAgentWnd;
}

// CCygSockChannel member functions

void CCygSockChannel::OnReceive(int nErrorCode)
{
	ASSERT( m_pAgentWnd != NULL );
	ASSERT( m_sCygSock != NULL );

	// TODO: client might not send everything in one shot
	// So this function should buffer anything until all data is received
	// Same for sending
	// For now, let's assume that all data can be sent in one chunk
	if(nErrorCode == 0)
	{
		int ret = -1;
		int out[4] = { 0, 0, 0, 0 };
		CString strDebug;

		switch(m_iState)
		{
			case 0: // af_local_recv_secret
				ret = Receive((char*)out, sizeof(out));
				if(ret != sizeof(out))
				{
					TRACE(_T("CCygSockChannel::OnReceive: af_local_recv_secret Receive failed. Closing socket\n"));
					Close();
					return;
				}
				TRACE(_T("CCygSockChannel::OnReceive: af_local_recv_secret. secret: %08x-%08x-%08x-%08x\n"), out[0], out[1], out[2], out[3]);
				if(this->m_sCygSock->CheckSecret(out))
				{
					// af_local_send_secret
					this->m_sCygSock->GetSecret(out);
					ret = Send(out, sizeof(out));
					if(ret != sizeof(out))
					{
						TRACE(_T("CCygSockChannel::OnReceive: af_local_send_secret Send failed. Closing socket\n"));
						Close();
						return;
					}
					TRACE(_T("CCygSockChannel::OnReceive: af_local_send_secret. Now in state 1\n"));
					m_iState = 1;
				}
				else
				{
					TRACE(_T("CCygSockChannel::OnReceive: af_local_recv_secret: SECRET MISMATCH\n"));
					Close();
					return;
				}
				break;
			case 1: // af_local_recv_cred
				m_peerCreds.pid = 0;
				m_peerCreds.uid = -1;
				m_peerCreds.gid = -1;

				ret = Receive(&m_peerCreds, sizeof(m_peerCreds));
				if(ret != sizeof(m_peerCreds))
				{
					TRACE(_T("CCygSockChannel::OnReceive: af_local_recv_cred Receive failed. Closing socket\n"));
					Close();
					return;
				}

				TRACE(_T("CCygSockChannel::OnReceive: af_local_recv_cred: eid credentials: pid: %d, uid: %d, gid: %d\n"),
					m_peerCreds.pid, m_peerCreds.uid, m_peerCreds.gid);

				{
					// af_local_send_cred
					SCygSock_PeerInfo myInfo;
					myInfo.pid = GetCurrentProcessId();
					myInfo.uid = m_peerCreds.uid;
					myInfo.gid = m_peerCreds.gid;

					if(Send(&myInfo, sizeof(myInfo)) != sizeof(myInfo))
					{
						TRACE(_T("CCygSockChannel::OnReceive: af_local_send_cred Send failed. Closing socket\n"));
						Close();
						return;
					}

					TRACE(_T("CCygSockChannel::OnReceive: af_local_send_cred. Now in state 2\n"));
				}

				m_iState = 2;
				break;
			default:
				if(m_pAgentWnd)
				{
					TRACE(_T("CCygSockChannel::OnReceive: Handing over to agent window...\n"));
					this->m_pAgentWnd->OnCygReceive(this);
				}
		}
	}
	CAsyncSocket::OnReceive(nErrorCode);
}

void CCygSockChannel::OnClose(int nErrorCode)
{
	CAsyncSocket::OnClose(nErrorCode);

	TRACE(_T("CCygSockChannel::OnClose: delete this (handle %x)\n"), this->m_hSocket);

	delete this;
}
