/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2011 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "KeePassExtensions.h"
#include "PuttyAgent.h"
#include "BalloonEvent.h"

CCriticalSection CKeePassExtensions::c_mapProtection;
std::map<HWND,CKeePassExtensions*> CKeePassExtensions::c_mapWindows;

CKeePassExtensions::CKeePassExtensions(HWND hWnd)
	: m_hWnd(hWnd)
	, m_BalloonEvent()
	, g_pOldWndProc(0)
{
	CSingleLock lock(&CKeePassExtensions::c_mapProtection, TRUE);
	CKeePassExtensions::c_mapWindows[hWnd] = this;
}

CKeePassExtensions::~CKeePassExtensions()
{
	CSingleLock lock(&CKeePassExtensions::c_mapProtection, TRUE);
	CKeePassExtensions::c_mapWindows.erase(this->m_hWnd);
}

LRESULT CKeePassExtensions::KeePassWinProc(HWND hWnd, UINT msg, WPARAM wParam, LPARAM lParam)
{
	CSingleLock lock(&CKeePassExtensions::c_mapProtection, TRUE);
	CKeePassExtensions *pInstance = CKeePassExtensions::c_mapWindows[hWnd];

	if(!pInstance)
		return 0;

	if(msg == WM_MY_SYSTRAY_NOTIFY)
	{
		switch(lParam)
		{
			case NIN_BALLOONTIMEOUT:
			case NIN_BALLOONHIDE:
				pInstance->m_BalloonEvent.SetBalloonDiscard();
				break;
			case NIN_BALLOONUSERCLICK:
				pInstance->m_BalloonEvent.SetBalloonClick();
				break;
		}
	}
	return pInstance->g_pOldWndProc(hWnd, msg, wParam, lParam);
}

BOOL CKeePassExtensions::Hook()
{
#pragma warning(disable:4244 4312)
	g_pOldWndProc = (WNDPROC)SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)KeePassWinProc);
#pragma warning(default:4244 4312)
	// TODO: return value!
	return TRUE;
}

BOOL CKeePassExtensions::Unhook()
{
#pragma warning(disable:4244)
	SetWindowLongPtr(m_hWnd, GWLP_WNDPROC, (LONG_PTR)g_pOldWndProc);
#pragma warning(default:4244)
	// TODO: return value!
	return TRUE;
}

BOOL CKeePassExtensions::ShowBalloon(const LPCTSTR info, const LPCTSTR title, UINT timeout /*= 5000*/, DWORD flags /*= NIIF_INFO*/)
{
	NOTIFYICONDATA tnd;
	memset(&tnd, 0, sizeof(tnd));

	tnd.cbSize = sizeof(tnd);
	tnd.hWnd = m_hWnd;
	tnd.uID = IDR_SYSTRAY_MENU;
	tnd.uFlags = NIF_INFO;

	strncpy_s(tnd.szInfoTitle, 63, title, 63);
	strncpy_s(tnd.szInfo, 255, info, 255);
	tnd.uTimeout = timeout;
	tnd.dwInfoFlags = flags;

	return Shell_NotifyIcon(NIM_MODIFY, &tnd);
}

BOOL CKeePassExtensions::ShowBalloonAndWaitForAnswer(const LPCTSTR info, const LPCTSTR title, UINT timeout /*= 5000*/, DWORD flags /*= NIIF_INFO*/)
{
	if(!Hook())
		return FALSE;

	BOOL bBallonSuccess = ShowBalloon(info, title, timeout, flags);

	if(bBallonSuccess) {
		m_BalloonEvent.WaitForBalloonEvent();
	}

	Unhook();

	BOOL bShallUnlock = m_BalloonEvent.ShallUnlock();
	m_BalloonEvent.Reset();

	// TODO: additional check by querying shellapi version

	if(!bBallonSuccess || bShallUnlock) {
		return TRUE;
	}

	return FALSE;
}

void CKeePassExtensions::UnlockKeepass()
{
	DWORD lThreadForeWin = GetCurrentThreadId();
	DWORD lThreadWindow = ::GetWindowThreadProcessId(m_hWnd, NULL);

	AttachThreadInput(lThreadForeWin, lThreadWindow, TRUE);
	SetForegroundWindow(m_hWnd);
	AttachThreadInput(lThreadForeWin, lThreadWindow, FALSE);

	SendMessage(m_hWnd, WM_COMMAND, ID_FILE_LOCK, 0);
}








