/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2011 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#pragma once

#include <afxtempl.h>

#include "PageantWnd.h"
#include "PageantThread.h"
#include "Singleton.h"

extern "C" {
#include "putty/tree234.h"
#include "putty/ssh.h"
#include "putty_additional.h"
}

#define PASSPHRASE_MAXLEN 512
#define AGENT_MAX_MSGLEN  8192

struct BLOBSTRUCT {
		unsigned char *blob;
		int len;
};

typedef CArray<CString> CKeyList;

class CKeyHandler : public Singleton<CKeyHandler>
{
friend class Singleton<CKeyHandler>;
protected:
	CKeyHandler();
	virtual ~CKeyHandler();

private:	
	CCriticalSection m_CritSection;

	CPageantWnd *m_pWindow;
	CPageantThread *m_pThread;

	tree234 *rsakeys;
	tree234 *ssh2keys;

	static int cmpkeys_rsa(void *av, void *bv);
	static int cmpkeys_ssh2(void *av, void *bv);
	static int cmpkeys_ssh2_asymm(void *av, void *bv);

	void *make_keylist1(int *length);
	void *make_keylist2(int *length);

public:
	int MessageHandler(unsigned char *msg);
	void DeleteKeys();
	BOOL AddKey(const char *data, int len, const char *pass, CString &err);
	void ListKeys(CKeyList &keys);
};
