/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2011 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "StdAfx.h"
#include <aclapi.h>
#include "PuttyAgent.h"
#include "PageantWnd.h"
#include "KeyHandler.h"
#include "KeePassExtensions.h"
#include "Utils.h"

#include "CygSockChannel.h"

extern IKpAPI* g_pAPI;

// TODO: Get from g_pAPI in future versions
// (does only work for current thread in
// current implementation)
extern HWND g_hKeePass;

IMPLEMENT_DYNAMIC(CPageantWnd, CFrameWnd)
CPageantWnd::CPageantWnd()
	: m_bConfBalloonReq(CONF_BALOON_REQ_DEF)
	, m_bConfBalloonInfo(CONF_BALOON_INFO_DEF)
	, m_bConfCygEnable(CONF_CYG_ENABLE_DEF)
{
	ASSERT( g_pAPI );
	ASSERT( g_hKeePass );

	IKpConfig *pCnf = NULL;
	if(g_pAPI->CreateInstance(CLSID_KpConfig, IID_IKpConfig, (void**)&pCnf) == S_OK)
	{
		m_bConfBalloonReq  = pCnf->GetBool(CONF_BALOON_REQ, CONF_BALOON_REQ_DEF);
		m_bConfBalloonInfo = pCnf->GetBool(CONF_BALOON_INFO, CONF_BALOON_INFO_DEF);
		m_bConfCygEnable = pCnf->GetBool(CONF_CYG_ENABLE, CONF_CYG_ENABLE_DEF);
		pCnf->Get(CONF_CYG_SOCKPATH, m_strConfSockPath.GetBuffer(SI_REGSIZE));
		m_strConfSockPath.ReleaseBuffer();		

		pCnf->Release();
		pCnf = NULL;
	}

	WNDCLASS wndcls;
	memset(&wndcls, 0, sizeof(WNDCLASS));

	wndcls.lpfnWndProc = ::DefWindowProc;
	wndcls.hInstance = AfxGetInstanceHandle();
	wndcls.lpszMenuName = NULL;
	wndcls.lpszClassName = PUTTY_APPNAME;

	AfxRegisterClass(&wndcls);

	Create(PUTTY_APPNAME, PUTTY_APPNAME, WS_DISABLED | WS_OVERLAPPEDWINDOW | WS_VSCROLL, CRect(100,100,100,100));
	ShowWindow(SW_HIDE);

	m_sCygSock.SetAgentWnd(this);
}

CPageantWnd::~CPageantWnd()
{
}

BEGIN_MESSAGE_MAP(CPageantWnd, CWnd)
	ON_WM_COPYDATA()
	ON_WM_CLOSE()
END_MESSAGE_MAP()

void CPageantWnd::CreateCygSocket()
{
	if(m_bConfCygEnable)
	{
		if(!m_sCygSock.Create(m_strConfSockPath))
		{
			CString strErrMsg;
			strErrMsg.Format(_T("Unable to create cygwin agent socket file %s.\n"), m_strConfSockPath);
			strErrMsg += _T("Possible Causes: (a) File exists (remove file and restart KeePass if it is not used).\n");
			strErrMsg += _T("(b) Cannot write to file or directory.\n");
			strErrMsg += _T("(c) Cannot create/listen socket on 127.0.0.1.\n");
			strErrMsg += _T("Cygwin agent functionality will be disabled.");
			::MessageBox(NULL, strErrMsg, _T("KeePass PuttyAgent plugin"), MB_ICONERROR | MB_OK);
			return;
		}
	}
}

// TODO: Actually, this stuff belongs into CKeyHandler::MessageHandler and should
// respond differently to different agent messages
BOOL CPageantWnd::UnlockKeePass(const CString &strMethod, const CString &strModuleName)
{
	CWnd *pCurrentWnd = CWnd::GetForegroundWindow();

	CKeePassExtensions kp(g_hKeePass);

	if(g_pAPI->IsFileOpen())
	{
		if(m_bConfBalloonInfo) {
			CString strBalloonMsg;
			if(!strModuleName.IsEmpty()) {
				strBalloonMsg = _T("Authenticating ");
				strBalloonMsg += strModuleName;
				strBalloonMsg += _T(" with SSH public key...");
			}
			else {
				strBalloonMsg = _T("Authentication with SSH public key...");
			}

			CString strBalloonTitle;
			strBalloonTitle.Format(_T("SSH pubkey authentication [%s]"), strMethod);
			kp.ShowBalloon(strBalloonMsg, strBalloonTitle, 3000);
		}
		return TRUE;
	}

	BOOL bUnlockRequest = TRUE;

	if(m_bConfBalloonReq)
	{
		CString strBalloonMsg;
		strBalloonMsg.Format(_T("[%s] "), strMethod);
		if(!strModuleName.IsEmpty()) {
			strBalloonMsg += strModuleName;
			strBalloonMsg += _T(" requested SSH pubkey authentication");
		}
		else {
			strBalloonMsg += _T("An application requested SSH pubkey authentication");
		}
		strBalloonMsg += _T("\nClick here to unlock workspace...");
		bUnlockRequest = kp.ShowBalloonAndWaitForAnswer(strBalloonMsg, _T("KeePass password required"));
	}

	if(bUnlockRequest) {
		kp.UnlockKeepass();
	}

	if(pCurrentWnd)
	{
		// TODO:
		// Improve. Sometimes it works, sometimes the window
		// flashes only in task bar
		::SetForegroundWindow(pCurrentWnd->m_hWnd);
	}

	return g_pAPI->IsFileOpen();
}

/**
 * This function is taken from WINPGNT.C
 */
PSID CPageantWnd::get_default_sid()
{
	HANDLE proc = NULL;
	DWORD sidlen;
	PSECURITY_DESCRIPTOR psd = NULL;
	PSID sid = NULL, copy = NULL, ret = NULL;
	
	if((proc = OpenProcess(MAXIMUM_ALLOWED, FALSE, GetCurrentProcessId())) == NULL)
		goto cleanup;
	
	if(GetSecurityInfo(proc, SE_KERNEL_OBJECT, OWNER_SECURITY_INFORMATION, &sid, NULL, NULL, NULL, &psd) != ERROR_SUCCESS)
		goto cleanup;
	
	sidlen = GetLengthSid(sid);
	copy = (PSID)smalloc(sidlen);
	
	if(!CopySid(sidlen, copy, sid))
		goto cleanup;

    /* Success. Move sid into the return value slot, and null it out
     * to stop the cleanup code freeing it. */
	ret = copy;
	copy = NULL;

cleanup:
	if(proc != NULL)
		CloseHandle(proc);
	if(psd != NULL)
		LocalFree(psd);
	if(copy != NULL)
		sfree(copy);
	
	return ret;
}

/**
 * This function is taken from WINPGNTC.C
 */
PSID CPageantWnd::get_user_sid()
{
	HANDLE proc = NULL, tok = NULL;
	TOKEN_USER *user = NULL;
	DWORD toklen, sidlen;
	PSID sid = NULL, ret = NULL;
	
	if((proc = OpenProcess(MAXIMUM_ALLOWED, FALSE, GetCurrentProcessId())) == NULL)
		goto cleanup;
	
	if(!OpenProcessToken(proc, TOKEN_QUERY, &tok))
		goto cleanup;
	
	if(!GetTokenInformation(tok, TokenUser, NULL, 0, &toklen) && GetLastError() != ERROR_INSUFFICIENT_BUFFER)
		goto cleanup;
	
	if((user = (TOKEN_USER *)LocalAlloc(LPTR, toklen)) == NULL)
		goto cleanup;
	
	if(!GetTokenInformation(tok, TokenUser, user, toklen, &toklen))
		goto cleanup;
	
	sidlen = GetLengthSid(user->User.Sid);
	
	sid = (PSID)smalloc(sidlen);
	
	if(!CopySid(sidlen, sid, user->User.Sid))
		goto cleanup;
	
	/* Success. Move sid into the return value slot, and null it out
	 * to stop the cleanup code freeing it. */
	ret = sid;
	sid = NULL;

cleanup:
	if(proc != NULL)
		CloseHandle(proc);
	if(tok != NULL)
		CloseHandle(tok);
	if(user != NULL)
		LocalFree(user);
	if(sid != NULL)
		sfree(sid);

	return ret;
}

BOOL CPageantWnd::OnCopyData(CWnd* pWnd, COPYDATASTRUCT* cds)
{
	char *mapname;
	unsigned char *p;
	HANDLE filemap;
	PSID mapowner = NULL, procowner = NULL, procowner2 = NULL;
	PSECURITY_DESCRIPTOR psd = NULL;
	BOOL ret = FALSE;

	if(cds->dwData != AGENT_COPYDATA_ID)
		return FALSE;

	mapname = (char *) cds->lpData;
	if(mapname[cds->cbData - 1] != '\0') {
		return FALSE;
	}

	CString strModuleName;
	if(m_bConfBalloonInfo || m_bConfBalloonReq) {
		Utils::GetModuleNameForFileMap(mapname, strModuleName);
	}
	if(!UnlockKeePass(_T("putty"), strModuleName)) {
		return FALSE;
	}

	filemap = OpenFileMapping(FILE_MAP_ALL_ACCESS, FALSE, mapname);

	if(filemap == NULL || filemap == INVALID_HANDLE_VALUE) {
		return FALSE;
	}

	if((procowner = get_user_sid()) == NULL) {
		goto cleanup;
	}
	if((procowner2 = get_default_sid()) == NULL) {
		goto cleanup;
	}

	if(GetSecurityInfo(filemap, SE_KERNEL_OBJECT, OWNER_SECURITY_INFORMATION, &mapowner, NULL, NULL, NULL, &psd) != ERROR_SUCCESS) {
		goto cleanup;
	}

	if(!EqualSid(mapowner, procowner) && !EqualSid(mapowner, procowner2)) {
		goto cleanup;
	}

	p = (unsigned char*)MapViewOfFile(filemap, FILE_MAP_WRITE, 0, 0, 0);
	if(!p)
		goto cleanup;

	CKeyHandler::GetInstance().MessageHandler(p);

	ret = TRUE;
	UnmapViewOfFile(p);

cleanup:
	if(procowner)
		sfree(procowner);
	if(procowner2)
		sfree(procowner2);
	if(psd)
		LocalFree(psd);
	CloseHandle(filemap);
	return ret;
}

void CPageantWnd::OnCygReceive(CCygSockChannel *sock)
{
	unsigned char p[AGENT_MAX_MSGLEN];
	memset(p, 0, sizeof(p));

	CString strModuleName;
	if(m_bConfBalloonInfo || m_bConfBalloonReq) {
		Utils::GetModuleNameForPid(sock->m_peerCreds.pid, strModuleName);
	}
	if(!UnlockKeePass(_T("cygwin"), strModuleName)) {
		TRACE(_T("CPageantWnd::OnCygReceive: Did not unlock workspace, closing socket\n"));
		sock->Close();
		return;
	}

	int nRead;
	nRead = sock->Receive(p, AGENT_MAX_MSGLEN);
	if(nRead == SOCKET_ERROR)
	{
		TRACE(_T("CPageantWnd::OnCygReceive: error on Receive, code: 0x%04X\n"), GetLastError());
		sock->Close();
		return;
	}

	TRACE(_T("CPageantWnd::OnCygReceive: Successfully received %d bytes, handing over to key handler\n"), nRead);

	int nSend = CKeyHandler::GetInstance().MessageHandler(p);

	if(sock->Send(p, nSend) != nSend)
	{
		TRACE(_T("CPageantWnd::OnCygReceive: Could not send %d bytes. Closing connection\n"), nSend);
		sock->Close();
		return;
	}
	TRACE(_T("CPageantWnd::OnCygReceive: Send successful. Sent %d bytes\n"), nSend);
}

void CPageantWnd::OnClose()
{
	PostQuitMessage(0);
	CFrameWnd::OnClose();
}


