/*
  PuttyAgent plugin for KeePass 1
  Copyright (C) 2011 Nikolaus Hammler <nikolaus@hammler.net>

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.

  You should have received a copy of the GNU General Public License
  along with this program; if not, write to the Free Software
  Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA
*/

#include "Utils.h"
#include <Psapi.h>
#include "SystemInfo.h"

CString Utils::GetWindowModuleName(CWnd *pWnd)
{
    HMODULE* lphModule;
    char FileName[1024];
    DWORD procid = 0;
    DWORD modulesize = 0;
    BOOL bInheritHandle = false;

    if(pWnd != NULL){
        GetWindowThreadProcessId(pWnd->m_hWnd,&procid);
        HANDLE process = OpenProcess(PROCESS_ALL_ACCESS | 
           PROCESS_QUERY_INFORMATION, bInheritHandle,procid);
        if(process != NULL){
            lphModule = new HMODULE[1];
            if(EnumProcessModules(process,lphModule, 
                      (sizeof(HMODULE)),&modulesize) != 0){
                GetModuleBaseName(process,lphModule[0],FileName,1024);
                CloseHandle(process);
                delete lphModule;
                return FileName;
            }
            delete lphModule;
        }
        CloseHandle(process);
    }
    return "";
}

BOOL Utils::GetModuleNameForFileMap(const CString &mapname, CString &dst)
{
	DWORD dwProcessID = 0;
	SystemHandleInformation hi;
	hi.SetFilter(_T("Section"), TRUE);
	if(hi.m_HandleInfos.GetHeadPosition() == NULL)
	{
		return FALSE;
	}

	CString strHandleName;
	for(POSITION pos = hi.m_HandleInfos.GetHeadPosition(); pos != NULL;)
	{
		SystemHandleInformation::SYSTEM_HANDLE& h = hi.m_HandleInfos.GetNext(pos);
		hi.GetName((HANDLE)h.HandleNumber, strHandleName, h.ProcessID);
		if(mapname == strHandleName.Right(mapname.GetLength()))
		{
			dwProcessID = h.ProcessID;
			break;
		}
	}

	if(!dwProcessID) {
		return FALSE;
	}

	if(!GetModuleNameForPid(dwProcessID, dst))
		return FALSE;

	return TRUE;
}

BOOL Utils::GetModuleNameForPid(DWORD dwProcessID, CString &dst)
{
	HANDLE processHandle = NULL;
	processHandle = OpenProcess(PROCESS_QUERY_INFORMATION | PROCESS_VM_READ, FALSE, dwProcessID);
	if(processHandle == NULL) {
		return FALSE;
	}

	DWORD dwRet = GetModuleFileNameEx(processHandle, NULL, dst.GetBuffer(MAX_PATH+1), MAX_PATH);
	dst.ReleaseBuffer(dwRet);
	CloseHandle(processHandle);

	if(dwRet == 0)
		return FALSE;

	return TRUE;
}


